TraceBw

Struct TraceBw 

pub struct TraceBw {
    pub pattern: Vec<(Duration, Vec<Bandwidth>)>,
    pub outer_index: usize,
    pub inner_index: usize,
}
This model is used to enable a more compact trace. It replays the bandwidth changes according to a trace file, and is necessary for replaying sampled traces from Internet or production.

Examples

use crate::netem_trace::model::Forever;
use netem_trace::model::BwTraceConfig;

let mut tracebw = TraceBwConfig::new().pattern(
    vec![
        (Duration::from_millis(1), vec![Bandwidth::from_mbps(2),Bandwidth::from_mbps(4),]),
        (Duration::from_millis(2), vec![Bandwidth::from_mbps(1)]),
       ]
).build();

assert_eq!(tracebw.next_bw(), Some((Bandwidth::from_mbps(2), Duration::from_millis(1))));
assert_eq!(tracebw.next_bw(), Some((Bandwidth::from_mbps(4), Duration::from_millis(1))));
assert_eq!(tracebw.next_bw(), Some((Bandwidth::from_mbps(1), Duration::from_millis(2))));
assert_eq!(tracebw.next_bw(), None);
assert_eq!(tracebw.next_bw(), None);
assert_eq!(tracebw.next_bw(), None);

let repeated_tracebw_config = TraceBwConfig::new().pattern(
    vec![
        (Duration::from_millis(1), vec![Bandwidth::from_mbps(2),Bandwidth::from_mbps(4),]),
        (Duration::from_millis(2), vec![Bandwidth::from_mbps(1)]),
       ]
).forever();

let mut repeated_tracebw = repeated_tracebw_config.clone().build();

assert_eq!(repeated_tracebw.next_bw(), Some((Bandwidth::from_mbps(2), Duration::from_millis(1))));
assert_eq!(repeated_tracebw.next_bw(), Some((Bandwidth::from_mbps(4), Duration::from_millis(1))));
assert_eq!(repeated_tracebw.next_bw(), Some((Bandwidth::from_mbps(1), Duration::from_millis(2))));

assert_eq!(repeated_tracebw.next_bw(), Some((Bandwidth::from_mbps(2), Duration::from_millis(1))));
assert_eq!(repeated_tracebw.next_bw(), Some((Bandwidth::from_mbps(4), Duration::from_millis(1))));
assert_eq!(repeated_tracebw.next_bw(), Some((Bandwidth::from_mbps(1), Duration::from_millis(2))));

assert_eq!(repeated_tracebw.next_bw(), Some((Bandwidth::from_mbps(2), Duration::from_millis(1))));
assert_eq!(repeated_tracebw.next_bw(), Some((Bandwidth::from_mbps(4), Duration::from_millis(1))));
assert_eq!(repeated_tracebw.next_bw(), Some((Bandwidth::from_mbps(1), Duration::from_millis(2))));

let ser : Box<dyn BwTraceConfig> = Box::new(repeated_tracebw_config);
let ser_str = serde_json::to_string(&ser).unwrap();

let json_str = "{\"RepeatedBwPatternConfig\":{\"pattern\":[{\"TraceBwConfig\":{\"pattern\":[[\"1ms\",[\"2Mbps\",\"4Mbps\"]],[\"2ms\",[\"1Mbps\"]]]}}],\"count\":0}}";
// The json string would be "{\"RepeatedBwPatternConfig\":{\"pattern\":[{\"TraceBwConfig\":{\"pattern\":[[{\"secs\":0,\"nanos\":1000000},[{\"gbps\":0,\"bps\":2000000},{\"gbps\":0,\"bps\":4000000}]],[{\"secs\":0,\"nanos\":2000000},[{\"gbps\":0,\"bps\":1000000}]]]}}],\"count\":0}}"
// if the `human` feature is not enabled.
#[cfg(not(feature = "human"))]
let json_str = "{\"RepeatedBwPatternConfig\":{\"pattern\":[{\"TraceBwConfig\":{\"pattern\":[[{\"secs\":0,\"nanos\":1000000},[{\"gbps\":0,\"bps\":2000000},{\"gbps\":0,\"bps\":4000000}]],[{\"secs\":0,\"nanos\":2000000},[{\"gbps\":0,\"bps\":1000000}]]]}}],\"count\":0}}";
assert_eq!(ser_str, json_str);

let des: Box<dyn BwTraceConfig> = serde_json::from_str(json_str).unwrap();
let mut model = des.into_model();

assert_eq!(model.next_bw(), Some((Bandwidth::from_mbps(2), Duration::from_millis(1))));
assert_eq!(model.next_bw(), Some((Bandwidth::from_mbps(4), Duration::from_millis(1))));
assert_eq!(model.next_bw(), Some((Bandwidth::from_mbps(1), Duration::from_millis(2))));

assert_eq!(model.next_bw(), Some((Bandwidth::from_mbps(2), Duration::from_millis(1))));
assert_eq!(model.next_bw(), Some((Bandwidth::from_mbps(4), Duration::from_millis(1))));
assert_eq!(model.next_bw(), Some((Bandwidth::from_mbps(1), Duration::from_millis(2))));

Fields

pattern: Vec<(Duration, Vec<Bandwidth>)>outer_index: usizeinner_index: usize