pub struct UniformDelayPerPacket<Rng = StdRng>where
Rng: RngCore,{
pub upper_bound: Delay,
pub lower_bound: Delay,
pub seed: u64,
pub count: usize,
/* private fields */
}Expand description
The model of a per-packet delay trace subjects to a uniform distribution.
The delay will subject to a uniform distribution in [lower_bound, upper_bound]
If the count is 0, the delay will be repeated forever, else it will be repeated count times.
Examples
Default values for lower_bound and upper_bound are 0ms and 10ms, respectively.
let mut uniform_delay = UniformDelayPerPacketConfig::new()
.upper_bound(Delay::from_micros(12100))
.lower_bound(Delay::from_micros(11900))
.count(2)
.seed(42)
.build();
assert_eq!(uniform_delay.next_delay(), Some(Delay::from_nanos(12005311)));
assert_eq!(uniform_delay.next_delay(), Some(Delay::from_nanos(12008545)));
assert_eq!(uniform_delay.next_delay(), None);Fields
upper_bound: Delaylower_bound: Delayseed: u64count: usizeTrait Implementations
Sourceimpl<Rng> Clone for UniformDelayPerPacket<Rng>
impl<Rng> Clone for UniformDelayPerPacket<Rng>
Sourcefn clone(&self) -> UniformDelayPerPacket<Rng>
fn clone(&self) -> UniformDelayPerPacket<Rng>
Returns a duplicate of the value. Read more
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl<Rng> Debug for UniformDelayPerPacket<Rng>
impl<Rng> Debug for UniformDelayPerPacket<Rng>
Sourceimpl<Rng: RngCore + Send> DelayPerPacketTrace for UniformDelayPerPacket<Rng>
impl<Rng: RngCore + Send> DelayPerPacketTrace for UniformDelayPerPacket<Rng>
fn next_delay(&mut self) -> Option<Delay>
Sourceimpl<Rng> PartialEq for UniformDelayPerPacket<Rng>
impl<Rng> PartialEq for UniformDelayPerPacket<Rng>
impl<Rng> Copy for UniformDelayPerPacket<Rng>
impl<Rng> StructuralPartialEq for UniformDelayPerPacket<Rng>where
Rng: RngCore,