pub struct RepeatedRwndPattern {
pub pattern: Vec<Box<dyn RwndTraceConfig>>,
pub count: usize,
/* private fields */
}Expand description
The model contains an array of rwnd trace models.
Combine multiple rwnd trace models into one rwnd pattern,
and repeat the pattern for count times.
If count is 0, the pattern will be repeated forever.
Examples
The most common use case is to read from a configuration file and
deserialize it into a RepeatedRwndPatternConfig:
let config_file_content = "{\"RepeatedRwndPatternConfig\":{\"pattern\":[{\"StaticRwndConfig\":{\"duration\":{\"secs\":1,\"nanos\":0},\"set_rcv_buf\":65536,\"app_read_bytes\":1024}},{\"StaticRwndConfig\":{\"duration\":{\"secs\":1,\"nanos\":0},\"rwnd_remaining\":32768}}],\"count\":2}}";
let des: Box<dyn RwndTraceConfig> = serde_json::from_str(config_file_content).unwrap();
let mut model = des.into_model();
let (decision, _) = model.next_rwnd().unwrap();
assert_eq!(decision.action, Some(RwndAction::AppRead { bytes: 1024 }));Fields
pattern: Vec<Box<dyn RwndTraceConfig>>count: usizeTrait Implementations
Sourceimpl RwndTrace for RepeatedRwndPattern
impl RwndTrace for RepeatedRwndPattern