DuplicateTrace

Trait DuplicateTrace 

pub trait DuplicateTrace: Send {
    // Required method
    fn next_duplicate(&mut self) -> Option<(DuplicatePattern, Duration)>;
}
This is a trait that represents a trace of duplicate patterns.

The trace is a sequence of (duplicate_pattern, duration) pairs. The duplicate_pattern describes how packets are duplicated when going through. The duration is the time that the duplicate_pattern lasts.

The next_duplicate function either returns the next duplicate_pattern and its duration in the sequence, or None if the trace goes to end.

Required Methods

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors