Module netem_trace::model
source · Expand description
This module contains pre-defined models for BwTrace, DelayTrace and LossTrace.
A model has two parts: a configuration struct and a model struct.
The configuration struct is used to configure the model and
used for serialization/deserialization if serde feature is enabled.
The model struct which implements trait BwTrace, DelayTrace or LossTrace
is used to generate the trace and maintain inner states.
Enable bw-model feature to use the BwTrace models.
Enable delay-model feature to use the DelayTrace models.
Enable loss-model feature to use the LossTrace models.
Re-exports
pub use bw::BwTraceConfig;pub use bw::NormalizedBwConfig;pub use bw::RepeatedBwPatternConfig;pub use bw::SawtoothBwConfig;pub use bw::StaticBwConfig;pub use bw::NormalizedBw;pub use bw::RepeatedBwPattern;pub use bw::SawtoothBw;pub use bw::StaticBw;pub use delay::DelayTraceConfig;pub use delay::RepeatedDelayPatternConfig;pub use delay::StaticDelayConfig;pub use delay::RepeatedDelayPattern;pub use delay::StaticDelay;pub use loss::LossTraceConfig;pub use loss::RepeatedLossPatternConfig;pub use loss::StaticLossConfig;pub use loss::RepeatedLossPattern;pub use loss::StaticLoss;
Modules
- This module contains some predefined bandwidth trace models.
- This module contains some predefined delay trace models.
- This module contains some predefined loss trace models.