Struct netem_trace::model::loss::RepeatedLossPattern
source · pub struct RepeatedLossPattern {
pub pattern: VecDeque<Box<dyn LossTrace>>,
}Expand description
The model contains an array of loss trace models.
Combines multiple loss trace models into one loss pattern,
and repeat the pattern for count times.
Examples
The most common use case is to read from a configuration file and
deserialize it into a RepeatedLossPatternConfig:
let config_file_content = "{\"RepeatedLossPatternConfig\":{\"pattern\":[{\"StaticLossConfig\":{\"loss\":[0.1,0.2],\"duration\":{\"secs\":1,\"nanos\":0}}},{\"StaticLossConfig\":{\"loss\":[0.2,0.4],\"duration\":{\"secs\":1,\"nanos\":0}}}],\"count\":2}}";
// The content would be "{\"RepeatedLossPatternConfig\":{\"pattern\":[{\"StaticLossConfig\":{\"loss\":[0.1,0.2],\"duration\":\"1s\"}},{\"StaticLossConfig\":{\"loss\":[0.2,0.4],\"duration\":\"1s\"}}],\"count\":2}}"
// if the `human` feature is enabled.
let des: Box<dyn LossTraceConfig> = serde_json::from_str(config_file_content).unwrap();
let mut model = des.into_model();
assert_eq!(
model.next_loss(),
Some((vec![0.1, 0.2], Duration::from_secs(1)))
);
assert_eq!(
model.next_loss(),
Some((vec![0.2, 0.4], Duration::from_secs(1)))
);
assert_eq!(
model.next_loss(),
Some((vec![0.1, 0.2], Duration::from_secs(1)))
);
assert_eq!(
model.next_loss(),
Some((vec![0.2, 0.4], Duration::from_secs(1)))
);
assert_eq!(model.next_loss(), None);You can also build manually:
let pat = vec![
Box::new(
StaticLossConfig::new()
.loss(vec![0.1, 0.2])
.duration(Duration::from_secs(1)),
) as Box<dyn LossTraceConfig>,
Box::new(
StaticLossConfig::new()
.loss(vec![0.2, 0.4])
.duration(Duration::from_secs(1)),
) as Box<dyn LossTraceConfig>,
];
let ser = Box::new(RepeatedLossPatternConfig::new().pattern(pat).count(2)) as Box<dyn LossTraceConfig>;
let ser_str = serde_json::to_string(&ser).unwrap();
let json_str = "{\"RepeatedLossPatternConfig\":{\"pattern\":[{\"StaticLossConfig\":{\"loss\":[0.1,0.2],\"duration\":{\"secs\":1,\"nanos\":0}}},{\"StaticLossConfig\":{\"loss\":[0.2,0.4],\"duration\":{\"secs\":1,\"nanos\":0}}}],\"count\":2}}";
// The json string would be "{\"RepeatedLossPatternConfig\":{\"pattern\":[{\"StaticLossConfig\":{\"loss\":[0.1,0.2],\"duration\":\"1s\"}},{\"StaticLossConfig\":{\"loss\":[0.2,0.4],\"duration\":\"1s\"}}],\"count\":2}}"
// if the `human` feature is enabled.
assert_eq!(ser_str, json_str);Fields
pattern: VecDeque<Box<dyn LossTrace>>Trait Implementations
sourceimpl LossTrace for RepeatedLossPattern
impl LossTrace for RepeatedLossPattern